@@ -1,8 +1,7 @@
 #!/usr/bin/env bash
 # lesspipe.sh, a preprocessor for less
-lesspipe_version=2.12
+lesspipe_version=2.13
 # Author: Wolfgang Friebel (wp.friebel AT gmail.com)
-#( [[ -n 1 && -n 2 ]] ) > /dev/null 2>&1 || exec zsh -y --ksh-arrays -- "$0" ${1+"$@"}
 
 has_cmd () {
 	command -v "$1" > /dev/null
@@ -11,9 +10,9 @@ has_cmd () {
 fileext () {
 	fn=${1##*/}
 	case "$fn" in
-		.*.*) extension=${fn##*.};;
-		.*) extension=;;
-		*.*) extension=${fn##*.};;
+		.*.*) extension=${fn##*.} ;;
+		.*) extension= ;;
+		*.*) extension=${fn##*.} ;;
 	esac
 	echo "$extension"
 }
@@ -75,12 +74,14 @@ filetype () {
 			[[ $ftype == json ]] && ftype=ipynb ;;
 		mp3)
 			[[ $ftype == mpeg ]] && ftype=mp3 ;;
-		jsx)
-			[[ $fcat == text ]] && ftype=jsx ;;
+		jsx|csv)
+			[[ $fcat == text ]] && ftype="$fext" ;;
 		tsx)
 			[[ $fcat == text ]] && ftype=typescript-jsx ;;
-		csv)
-			[[ $fcat == text ]] && ftype=csv ;;
+		appimage)
+			[[ $fcat == application && -x "$1" ]] && ftype="$fext" ;;
+		snap)
+			[[ $fcat == application ]] && ftype="$fext" ;;
 	esac
 	### get file type from 'file' command for an unspecific result
 	if [[ "$fcat" == message && $ftype == plain ]]; then
@@ -101,7 +102,7 @@ filetype () {
 			*PEM\ certificate\ request)
 				ftype=csr ;;
 			*PEM\ certificate)
-				ftype=csr ;;
+				ftype=x509 ;;
 			*Microsoft\ OOXML)
 				ftype=docx ;;
 			Apple\ binary\ property\ list)
@@ -133,28 +134,18 @@ filetype () {
 		case "$fext" in
 			crt|pem)
 				ftype=x509 ;;
-			crl)
-				ftype=crl ;;
-			csr)
-				ftype=csr ;;
+			crl|csr)
+				ftype="$fext" ;;
 		esac
 		if [[ $fchar != binary ]]; then
 		# text only file formats
 			case "$fext" in
-				pod)
-					ftype=pod ;;
+				html|htm|xml|pod|log)
+					ftype="$fext" ;;
 				pm)
 					ftype=perl ;;
-				crt|pem)
-					ftype=x509 ;;
-				crl)
-					ftype=crl ;;
-				csr)
-					ftype=csr ;;
 				md|MD|mkd|markdown|rst)
 					ftype=markdown ;;
-				log)
-					ftype=log ;;
 				ebuild|eclass)
 					ftype='sh' ;;
 			esac
@@ -370,18 +361,28 @@ get_unpack_cmd () {
 		ms-cab-compressed)
 			{ has_cmd bsdtar && prog=bsdtar; } ||
 			{ has_cmd cabextract && prog=cabextract; } ;;
-		7z-compressed)
-			{ has_cmd 7zz && prog=7zz; } ||
-			{ has_cmd 7zr && prog=7zr; } ||
-			{ has_cmd 7z && prog=7z; } ||
-			{ has_cmd 7za && prog=7za; } ;;
 		iso9660-image)
 			{ has_cmd bsdtar && prog=bsdtar; } ||
 			{ has_cmd isoinfo && prog=isoinfo; } ;;
+		cpio)
+			{ has_cmd cpio && prog=cpio; } ||
+			{ has_cmd bsdtar && prog=bsdtar; } ;;
 		archive)
 			prog='ar'
-			has_cmd bsdtar && prog=bsdtar
+			has_cmd bsdtar && prog=bsdtar ;;
+		appimage|snap)
+			has_cmd unsquashfs && cmd=(isimage "$x" "$2" "$file2") ;;
 	esac
+	# 7z formats and fall back to 7z supported formats
+	if [[ -z $prog ]]; then
+		case "$x" in
+			7z-compressed|lzma|xz|cab|arj|bzip2|cpio|iso)
+				{ has_cmd 7zz && prog=7zz; } ||
+				{ has_cmd 7zr && prog=7zr; } ||
+				{ has_cmd 7z && prog=7z; } ||
+				{ has_cmd 7za && prog=7za; } ;;
+		esac
+	fi
 	[[ -n $prog ]] && cmd=(isarchive "$prog" "$2" "$file2")
 	if [[ -n ${cmd[*]} ]]; then
 		[[ -n "$file2" ]] && file2= && return
@@ -411,12 +412,12 @@ analyze_args () {
 	COLOR="--color=auto"
 	has_cmd tput && colors=$(tput colors) || colors=0
 	if [[ $colors -ge 8 ]]; then
-		[[ $LESS =~ -[A-Za-z~]*[rR] || $i = --raw-control-chars || $i = --RAW-CONTROL-CHARS ]] && COLOR="--color=always"
+		lessarg="$LESS $lessarg"
 		# shellcheck disable=SC2206
 		r_string=($lessarg)
 		for i in "${r_string[@]}"
 		do
-			[[ $i =~ [\s]-[A-Za-z~]*[rR] || $i = --raw-control-chars || $i = --RAW-CONTROL-CHARS ]] && COLOR="--color=always"
+			[[ $i =~ ^-[A-Za-z~]*[rR] || $i = --raw-control-chars || $i = --RAW-CONTROL-CHARS ]] && COLOR="--color=always"
 		done
 	fi
 	# last argument starting with colon or equal sign is used for piping into less
@@ -473,6 +474,10 @@ has_colorizer () {
 
 isfinal () {
 	if [[ "$2" == *$sep ]]; then
+		if [[ "$2" == "$sep" && "$x" == html ]]; then
+			[[ $COLOR == *always ]] && colarg="--color" || colarg="--mono"
+			has_cmd xmq && isxmq "$1" html && return
+		fi
 		cat "$1"
 		return
 	fi
@@ -489,7 +494,11 @@ isfinal () {
 				fi
 			fi
 			msg="$x: showing the output of ${cmd[*]}" ;;
-		html|xml)
+		xml)
+			[[ -z $file2 ]] &&
+			{ { has_cmd xmq && cmd=(isxmq "$1" xml); } ||
+			{ has_htmlprog && cmd=(ishtml "$1"); }; } ;;
+		html)
 			[[ -z $file2 ]] && has_htmlprog && cmd=(ishtml "$1") ;;
 		dtb|dts)
 			has_cmd dtc && cmd=(isdtb "$1") ;;
@@ -516,50 +525,45 @@ isfinal () {
 					-o $t2" "$1" && cmd=(mdcat "$t2"); } ||
 				{ has_cmd pandoc && istemp "pptx2md --disable-image --disable-wmf \
 					-o $t2" "$1" && cmd=(pandoc -f markdown -t plain "$t2"); }; } ||
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" ppt); } ;;
+			{ can_do_office && cmd=(isoffice "$1" ppt); } ;;
 		xlsx|ods)
 			{ has_cmd xlscat && cmd=(istemp "xlscat -L -R all" "$1"); } ||
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" "$x"); } ;;
+			{ can_do_office && cmd=(isoffice "$1" "$x"); } ;;
 		odt)
 			{ has_cmd odt2txt && cmd=(istemp odt2txt "$1"); } ||
 			{ has_cmd pandoc && cmd=(pandoc -f odt -t plain "$1"); } ||
 			{ has_cmd libreoffice && cmd=(isoffice2 "$1"); } ;;
 		odp)
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" odp); } ;;
+			{ can_do_office && cmd=(isoffice "$1" odp); } ;;
 		msword)
 			t="$1"; [[ "$t" == - ]] && t=/dev/stdin
 			{ has_cmd wvText && cmd=(istemp wvText "$t" /dev/stdout); } ||
 			{ has_cmd catdoc && cmd=(catdoc "$1"); } ||
 			{ has_cmd libreoffice && cmd=(isoffice2 "$1"); } ;;
 		ms-powerpoint)
-			{ has_cmd broken_catppt && cmd=(istemp catppt "$1"); } ||
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" ppt); } ;;
+			{ can_do_office && cmd=(isoffice "$1" ppt); } ;;
 		ms-excel)
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" xls); } ;;
+			{ can_do_office && cmd=(isoffice "$1" xls); } ;;
 		ooffice1)
 			{ has_cmd sxw2txt && cmd=(istemp sxw2txt "$1"); } ||
-			{ has_cmd libreoffice && has_htmlprog && cmd=(isoffice "$1" odt); } ;;
+			{ can_do_office && cmd=(isoffice "$1" odt); } ;;
 		ipynb|epub)
 			has_cmd pandoc && cmd=(pandoc -f "$x" -t plain "$1") ;;
 		troff)
-			if has_cmd groff; then
-				fext=$(fileext "$1")
-				declare macro=andoc
-				[[ "$fext" == me ]] && macro=e
-				[[ "$fext" == ms ]] && macro=s
-				cmd=(groff -s -p -t -e -Tutf8 -m "$macro" "$1")
-			elif has_cmd mandoc; then
-				cmd=(mandoc -l "$1")
-			elif has_cmd man; then
-				cmd=(man -l "$1")
-			fi ;;
+			fext=$(fileext "$1")
+			declare macro=andoc
+			[[ "$fext" == me ]] && macro=e
+			[[ "$fext" == ms ]] && macro=s
+			{ has_cmd groff && cmd=(groff -s -p -t -e -Tutf8 -m "$macro" "$1"); } ||
+			{ has_cmd mandoc && cmd=(nodash mandoc "$1"); } ||
+			{ has_cmd man && cmd=(nodash man "$1"); } ;;
 		rtf)
 			{ has_cmd unrtf && cmd=(istemp "unrtf --text" "$1"); } ||
 			{ has_cmd libreoffice && cmd=(isoffice2 "$1"); } ;;
 		dvi)
 			has_cmd dvi2tty && cmd=(istemp "dvi2tty -q" "$1") ;;
 		sharedlib)
-			cmd=(istemp nm "$1");;
+			cmd=(istemp nm "$1") ;;
 		pod)
 			[[ -z $file2 ]] &&
 			{ { has_cmd pod2text && cmd=(pod2text "$1"); } ||
@@ -656,6 +660,12 @@ isarchive () {
 				istemp cabextract2 "$2" "$3" ;;
 			isoinfo)
 				istemp "isoinfo -i" "$2" "-x$3" ;;
+			cpio)
+				if [[ "$2" == - ]]; then
+					cpio -i --quiet --to-stdout "$3"
+				else
+					cpio -i --quiet --to-stdout --file "$2" "$3"
+				fi ;;
 			7zz|7za|7zr)
 				istemp "$prog e -so" "$2" "$3"
 		esac
@@ -678,6 +688,12 @@ isarchive () {
 				isoinfo -d -i "$t"| grep -E '^Joliet' && joliet=J
 				separatorline
 				isoinfo -fR"$joliet" -i "$t" ;;
+			cpio)
+				if [[ "$2" == - ]]; then
+					cpio -tv --quiet
+				else
+					cpio -tv --quiet --file "$2"
+				fi ;;
 			7zz|7za|7zr)
 				istemp "$prog l" "$2"
 		esac
@@ -693,6 +709,21 @@ ispdf () {
 	istemp pdftohtml -i -q -s -noframes -nodrm -stdout "$1"|ishtml -
 }
 
+isimage () {
+	if [[ "$1" == appimage ]]; then
+		offset="-o $("$2" --appimage-offset)"
+	fi
+	if [[ -z "$3" ]]; then
+		if [[ "$1" == snap ]]; then
+			has_cmd snap && snap info "$2"
+			separatorline
+		fi
+		istemp "unsquashfs -d . -llc $offset" "$2"
+	else
+		istemp "unsquashfs -cat $offset"  "$2" "$3"
+	fi
+}
+
 isrpm () {
 	if [[ -z "$2" ]]; then
 		if has_cmd rpm; then
@@ -755,6 +786,12 @@ isoffice2 () {
 	istemp "libreoffice --headless --cat" "$1" 2>/dev/null
 }
 
+isxmq () {
+	[[ $COLOR == *always ]] && colarg="--color" || colarg="--mono"
+	msg "xmq output, append :$2 to the filename to see the (colored) contents"
+	xmq "$1" render-terminal "$colarg"
+}
+
 isdtb () {
 	errors=$(nexttmp)
 	has_colorizer "-" "dts" "dts"
@@ -766,6 +803,13 @@ isdtb () {
 	fi
 }
 
+can_do_office () {
+	if has_htmlprog && has_cmd libreoffice; then
+		return 0
+	fi
+	return 1
+}
+
 has_htmlprog () {
 	if has_cmd w3m || has_cmd lynx || has_cmd elinks || has_cmd html2text; then
 		return 0
